# possbible extension: dimension -> dimensions
#
# /**
#
# \name{simpleApply}
#
# \title{simpleApply}
#
# \alias{simpleApply}
#
# \keyword{}
#
# \description{
#   Attention: Be careful with \code{funcResultDimensionality} !
#   \code{func} is applied to all subsets
#   of \code{arrayObject} defined by looping over
#   all indexes given in the dimension \code{dimension}.
#   \code{func} must be unary.  }
#
# \value{an array of dim=c(dim(arrayObject)[dimension], funcResultDimensionality); possibly use \code{\link{aperm}} to rearrange the dimensions}
#
#
# \arguments{
#  \item{arrayObject}{array object}
#  \item{dimension}{numeric}
#  \item{func}{unary function}
#  \item{funcResultDimensionality}{numeric (vector) specifying the dimensionality of the result of \code{func}}
# }
# 
# \details{}
#
# \seealso{\code{\link{}}}
#
# \examples{
#
#r <- simpleApply(matrix(c(1, 2)), 1, sum, 1)
#stopifnot( r == matrix(c(1,2)))

#a <- array(c(1:30),dim=c(3,2,5))
#r <- simpleApply(a, 1, function(x){return(x[1])}, 1)
#stopifnot( all(r == matrix(data=c(1:3))))

#r <- simpleApply(a, 1, function(x){return(x[2,5])}, 1)
#stopifnot( all(r == matrix(data=c(28:30))))

#r <- simpleApply(a, 1, function(x){return(x[1,3])}, 1)
#stopifnot( all(r == matrix(data=c(13:15))))

#r <- simpleApply(a, 1, function(x){return(x[,])}, c(2,5))
#stopifnot( all( a == r))

#r <- simpleApply(a, 2, function(x){return(x[,])}, c(3,5))
#stopifnot( all( a == aperm(r,c(2,1,3)) ) )

#r <- simpleApply(a, 3, function(x){return(x[,])}, c(3,2))
#stopifnot( all( a == aperm(r,c(2,3,1)) ) )

#r2 <- simpleApply(a, 1, function(x){ return(as.vector(x))}, 10)

#vec <- 1:10
#dim(vec) <- c(10,1)
#mat <- matrix(data=rep(1:10,4),nrow=10,ncol=4,byrow=FALSE)
#r <- simpleApply(mat,1,function(y){return(mean(y))},1)
#stopifnot(all(r==vec))
#
#	}
#
# \keyword{}
#
# \author{Andreas Buness <a.buness@dkfz.de>}
# */

simpleApply <- function(arrayObject, dimension, func, funcResultDimensionality, DEBUG=FALSE){


if( missing(funcResultDimensionality) ){
  stop(" funcResultDimensionality is missing in simpleApply ")
}
if( ! is.numeric(funcResultDimensionality) ){
  stop(" funcResultDimensionality must be numeric in simpleApply ")
}
if( missing(func) ){
  stop(" func is missing in simpleApply ")
}
if( missing(arrayObject) ){
  stop(" arrayObject is missing in simpleApply ")
}
if( missing(dimension) ){
  stop(" dimension is missing in simpleApply ")
}
if( ! is.array(arrayObject) ){
  stop(" arrayObject must be of type array in simpleApply ")
}
if( ! is.numeric(dimension) ){
  stop(" dimension must be of type numeric in simpleApply ")
}
if( (dimension < 1) | (dimension > length(dim(arrayObject)))    ){
  stop(" dimension must be valid/meaningful in simpleApply ")
}


result <- array(data=NA, dim=c(dim(arrayObject)[dimension],funcResultDimensionality))

nrOfDimensions <- length(dim(arrayObject))

arrayObjectString <- "arrayObject["
preString <- paste(paste(rep(",",dimension-1),collapse=""),sep="")
postString <- paste(paste(rep(",",nrOfDimensions-dimension),collapse=""),"]",sep="")

resultPreString <- "result["
resultPostString <- paste(paste(rep(",",length(funcResultDimensionality)),collapse=""),"] <- resultI",sep="")

if(DEBUG) print(" next run : ")

for( i in (1:dim(arrayObject)[dimension]) ){

  tmpObjectText <- paste(arrayObjectString,preString,i,postString,sep="")
  if(DEBUG) print(tmpObjectText)
  arrayObjectI <- eval(parse(text=tmpObjectText))

  resultI <- func( arrayObjectI )
  tmpResultText <- paste(resultPreString,i,resultPostString,sep="")
  if(DEBUG) print(tmpResultText)
  eval(parse(text=tmpResultText))
  
}

return(result)

}# end of simpleApply
